package ovh.herisson.Clyde.Services;

import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.CourseRepository;
import ovh.herisson.Clyde.Repositories.LessonRepository;
import ovh.herisson.Clyde.Tables.Course;
import ovh.herisson.Clyde.Tables.Lesson;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.User;

import java.lang.reflect.GenericSignatureFormatError;
import java.util.ArrayList;
import java.util.Map;

@Service
public class LessonService {
    private final LessonRepository lessonRepo;
    private final CourseRepository courseRepo;
    public LessonService(LessonRepository lessonRepo, CourseRepository courseRepo){
        this.lessonRepo = lessonRepo;
        this.courseRepo = courseRepo;
    }

    public Lesson save(Lesson lesson){
        return lessonRepo.save(lesson);
    }
    public Lesson findById(long id){
        return lessonRepo.findById(id);
    }
    public Iterable<Lesson> findAll(){return lessonRepo.findAll();}

    public Iterable<Lesson> findAllOwnedLesson(User teacher){
        ArrayList<Lesson> toReturn = new ArrayList<>();
        ArrayList<Course> coursesOwned = (ArrayList<Course>) courseRepo.findAllOwnedCoures(teacher);
        for (Course element : coursesOwned) {
            for(Lesson lesson : lessonRepo.findLessonByCourse(element))
            toReturn.add(lesson);
        }
        return toReturn;
    }

    public Lesson createLesson(Map<String,Object> lessonInfos) {
        Lesson target = new Lesson();

        for (Map.Entry<String, Object> entry : lessonInfos.entrySet()) {
            switch (entry.getKey()) {
                case "lessonStart":
                    target.setLessonStart((String) entry.getValue());
                    break;
                case "lessonEnd":
                    target.setLessonEnd((String) entry.getValue());
                case "color":
                    target.setColor((String) entry.getValue());
                    break;
                case "local":
                    target.setLocal((String) entry.getValue());
                    break;
                case "lessonType":
                    target.setLessonType((String) entry.getValue());
                    break;
                case "courseId":
                    target.setCourse(courseRepo.findById((int) entry.getValue()));
                    break;
            }

        }
        return target;
    }

    public boolean modifyData(long id, Map<String ,Object> updates){
        Lesson target = lessonRepo.findById(id);

        if(target == null)
            return false;

        for (Map.Entry<String , Object> entry: updates.entrySet()){
            switch (entry.getKey()){
                case "lessonStart":
                    target.setLessonStart((String) entry.getValue());
                    break;
                case "lessonEnd":
                    target.setLessonEnd((String) entry.getValue());
                    break;            
                case "local":
                    target.setLocal((String) entry.getValue());
                    break;
                case "lessonType":
                    target.setLessonType((String) entry.getValue());
                    break;
            }
        }
        lessonRepo.save(target);
        return true;
    }

    public void delete(Lesson lesson){
        lessonRepo.delete(lesson);
    }


}
